//+-------------------------------------------------------------------------------------+
//|                                                              Support&Resistance.mq4 |
//|                                                                           Scriptong |
//|                                                                   scriptong@mail.ru |
//+-------------------------------------------------------------------------------------+
#property copyright "Scriptong"
#property link      "scriptong@mail.ru"


//---- input parameters
extern double    Lots = 0.1;
extern string    OpenOrderSound = "ok.wav";
extern int       MagicNumber = 875;
extern color     SupportColor = Blue;                            //   
extern color     ResistanceColor = Red;                      //   

bool Activate, FreeMarginAlert, FatalError;
double Tick, Spread, StopLevel, FreezeLevel, MinLot, MaxLot, LotStep, Supp, Resis;
datetime LastBar, ChanStart;
string SName, RName;

//+-------------------------------------------------------------------------------------+
//| expert initialization function                                                      |
//+-------------------------------------------------------------------------------------+
int init()
  {
   FatalError = False;
   Activate = False;
// - 1 - ==      ========================================
   Tick = MarketInfo(Symbol(), MODE_TICKSIZE);                         //  
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                 //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //   
   FreezeLevel = ND(MarketInfo(Symbol(), MODE_FREEZELEVEL)*Point);   //  
   MinLot = MarketInfo(Symbol(), MODE_MINLOT);    //    
   MaxLot = MarketInfo(Symbol(), MODE_MAXLOT);   //    
   LotStep = MarketInfo(Symbol(), MODE_LOTSTEP);          //    
// - 1 - ==   =============================================================

// - 2 - ==          =======
   Lots = MathRound(Lots/LotStep)*LotStep; //     
   if(Lots < MinLot || Lots > MaxLot) //     MinLot    MaxLot
     {
      Comment(" Lots     !  !");
      return(0);
     }
// - 2 - ==   =============================================================

   SName = "Support"+MagicNumber;                    //    " "
   RName = "Resistance"+MagicNumber;             //    " "
    
   Activate = True;
   
//----
   return(0);
  }
//+-------------------------------------------------------------------------------------+
//| expert deinitialization function                                                    |
//+-------------------------------------------------------------------------------------+
int deinit()
  {
//----
   Comment("");
//----
   return(0);
  }
  
//+-------------------------------------------------------------------------------------+
//|                                                |
//+-------------------------------------------------------------------------------------+
double ND(double A)
{
 return(NormalizeDouble(A, Digits));
}  

//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
string ErrorToString(int Error)
{
 switch(Error)
   {
    case 2: return("  ,   ."); 
    case 5: return("    ,  ."); 
    case 6: return("   ,   ."); 
    case 64: return(" ,   .");
    case 132: return(" ."); 
    case 133: return(" ."); 
    case 149: return(" ."); 
   }
}

//+-------------------------------------------------------------------------------------+
//|   .   ,   True,  - False    |
//+-------------------------------------------------------------------------------------+  
bool WaitForTradeContext()
{
 int P = 0;
 //  ""
 while(IsTradeContextBusy() && P < 5)
   {
    P++;
    Sleep(1000);
   }
 // -------------  
 if(P == 5)
   return(False);
 return(True);    
}
  
//+-------------------------------------------------------------------------------------+
//| ""                                                         |
//|    OpenOrder            |
//| :                                                                         |
//|   0 -                                                                      |
//|   1 -                                                                 |
//|   2 -   Price                                                         |
//|   3 -   SL                                                            |
//|   4 -   TP                                                            |
//|   5 -   Lot                                                           |
//+-------------------------------------------------------------------------------------+
int OpenOrderCorrect(int Type, double Price, double SL, double TP, 
                     bool Redefinition = True)
// Redefinition -  True     
//                 False -  
{
// - 1 - ==     ====================================
 if(AccountFreeMarginCheck(Symbol(), OP_BUY, Lots) <= 0 || GetLastError() == 134) 
  {
   if(!FreeMarginAlert)
    {
     Print("    . Free Margin = ", 
           AccountFreeMargin());
     FreeMarginAlert = True;
    } 
   return(5);  
  }
 FreeMarginAlert = False;  
// - 1 - ==   =============================================================

// - 2 - ==   Price, SL  TP    ====================

 RefreshRates();
 switch (Type)
   {
    case OP_BUY: 
                string S = "BUY"; 
                if (MathAbs(Price-Ask)/Point > 3)
                  if (Redefinition) Price = ND(Ask);
                  else              return(2);
                if (ND(TP-Bid) < StopLevel && TP != 0)
                  if (Redefinition) TP = ND(Bid+StopLevel);
                  else              return(4);
                if (ND(Bid-SL) < StopLevel)
                  if (Redefinition) SL = ND(Bid-StopLevel);
                  else              return(3);
                break;
    case OP_SELL: 
                 S = "SELL"; 
                 if (MathAbs(Price-Bid)/Point > 3)
                   if (Redefinition) Price = ND(Bid);
                   else              return(2);
                 if (ND(Ask-TP) < StopLevel) 
                   if (Redefinition) TP = ND(Ask-StopLevel);
                   else              return(4);
                 if (ND(SL-Ask) < StopLevel && SL != 0)
                   if (Redefinition) SL = ND(Ask+StopLevel);
                   else              return(3);
                 break;
    case OP_BUYSTOP: 
                    S = "BUYSTOP";
                    if (ND(Price-Ask) < StopLevel)
                      if (Redefinition) Price = ND(Ask+StopLevel);
                      else              return(2);
                    if (ND(TP-Price) < StopLevel && TP != 0)
                      if (Redefinition) TP = ND(Price+StopLevel);
                      else              return(4);
                    if (ND(Price-SL) < StopLevel)
                      if (Redefinition) SL = ND(Price-StopLevel);
                      else              return(3);
                    break;
    case OP_SELLSTOP: 
                     S = "SELLSTOP";
                     if (ND(Bid-Price) < StopLevel)
                       if (Redefinition) Price = ND(Bid-StopLevel);
                       else              return(2);
                     if (ND(Price-TP) < StopLevel)
                       if (Redefinition) TP = ND(Price-StopLevel);
                       else              return(4);
                     if (ND(SL-Price) < StopLevel && SL != 0)
                       if (Redefinition) SL = ND(Price+StopLevel);
                       else              return(3);
                     break;
    case OP_BUYLIMIT: 
                     S = "BUYLIMIT";
                     if (ND(Ask-Price) < StopLevel)
                      if (Redefinition) Price = ND(Ask-StopLevel);
                      else              return(2);
                     if (ND(TP-Price) < StopLevel && TP != 0)
                       if (Redefinition) TP = ND(Price+StopLevel);
                       else              return(4);
                     if (ND(Price-SL) < StopLevel)
                       if (Redefinition) SL = ND(Price-StopLevel);
                       else              return(3);
                     break;
    case OP_SELLLIMIT: 
                     S = "SELLLIMIT";
                     if (ND(Price - Bid) < StopLevel) 
                       if (Redefinition) Price = ND(Bid+StopLevel);
                       else              return(2);
                     if (ND(Price-TP) < StopLevel)
                       if (Redefinition) TP = ND(Price-StopLevel);
                       else              return(4);
                     if (ND(SL-Price) < StopLevel && SL != 0)
                       if (Redefinition) SL = ND(Price+StopLevel);
                       else              return(3);
                     break;
   }
// - 2 - ==   =============================================================
 
// - 3 - ==       =================================
 if(WaitForTradeContext())  //    
   {  
    Comment("     ", S, " ...");  
    int ticket=OrderSend(Symbol(), Type, Lots, Price, 3, 
               SL, TP, NULL, MagicNumber, 0);//  
    //     
    if(ticket<0)
      {
       int Error = GetLastError();
       if(Error == 2 || Error == 5 || Error == 6 || Error == 64 
          || Error == 132 || Error == 133 || Error == 149)     //   
         {
          Comment("     . . "+
                   ErrorToString(Error)+"  !");
          FatalError = True;
         }
        else 
         Comment("   ", S, ": ", Error);       //  
       return(1);
      }
    // ---------------------------------------------
    
    //      
    Comment(" ", S, "  !"); 
    PlaySound(OpenOrderSound); 
    return(0); 
    // ------------------------
   }
  else
   {
    Comment("     !");
    return(1);  
   } 
// - 3 - ==   =============================================================
   
}

//+-------------------------------------------------------------------------------------+
//|                                              |
//+-------------------------------------------------------------------------------------+
void ShowChannel()
{
 if (Supp != 0 && Resis != 0)                                   //   
   {
    if (ObjectFind(SName) < 0)      //    ,    
      {
       ObjectCreate(SName, OBJ_TREND, 0, ChanStart, Supp, Time[0], Supp); 
       ObjectSet(SName, OBJPROP_RAY, False);
       ObjectSet(SName, OBJPROP_COLOR, SupportColor);
      }
     else      //     ,       
      {
       ObjectMove(SName, 0, ChanStart, Supp);
       ObjectMove(SName, 1, Time[0], Supp);
      } 
    if (ObjectFind(RName) < 0)   //   ,    
      {
       ObjectCreate(RName, OBJ_TREND, 0, ChanStart, Resis, Time[0], Resis);
       ObjectSet(RName, OBJPROP_RAY, False);
       ObjectSet(RName, OBJPROP_COLOR, ResistanceColor);
      }
     else  //     ,       
      {
       ObjectMove(RName, 0, ChanStart, Resis);
       ObjectMove(RName, 1, Time[0], Resis);
      } 
   }
  else           //    ,      
   {
    if (ObjectFind(SName) == 0)
      ObjectDelete(SName);
    if (ObjectFind(RName) == 0)
      ObjectDelete(RName);
   } 
}

//+-------------------------------------------------------------------------------------+
//|                                                 |
//+-------------------------------------------------------------------------------------+
void GetLevels()
{
 Supp = 0;
 Resis = 0;
// - 1 - ==     =================================================
 double AO1 = iAO(Symbol(), 0, 1);
 double AO2 = iAO(Symbol(), 0, 2); 
 double AO3 = iAO(Symbol(), 0, 3); 
// - 1 - ==   =============================================================

// - 2 - ==          ==========
 if (AO1 > 0)
   if (AO1 < AO2 && AO2 > AO3)                   //     
     {
      //  ,      
      int i = 4;   
      while (AO3 > 0 && i < Bars)
        {
         AO3 = iAO(Symbol(), 0, i);
         i++;
        }
      if (i == Bars) return;                //     ,   
      //  
      Supp = Low[iLowest(Symbol(), 0, MODE_LOW, i-1)];
      Resis = High[iHighest(Symbol(), 0, MODE_HIGH, i-1)];
      ChanStart = Time[i-2];              //      
     }
// - 2 - ==   =============================================================

// - 3 - ==          ==========
 if (AO1 < 0)
   if (AO1 > AO2 && AO2 < AO3)                      //     
     {
      //  ,      
      i = 4;
      while (AO3 < 0 && i < Bars)
        {
         AO3 = iAO(Symbol(), 0, i);
         i++;
        } 
      if (i == Bars) return;                //     ,   
      //  
      Supp = Low[iLowest(Symbol(), 0, MODE_LOW, i-1)];
      Resis = High[iHighest(Symbol(), 0, MODE_HIGH, i-1)];
      ChanStart = Time[i-2];              //      
     }
// - 3 - ==   =============================================================
 
 ShowChannel();                                                    //  
}

//+-------------------------------------------------------------------------------------+
//|                                                |
//+-------------------------------------------------------------------------------------+
double NP(double A)
{
 return(MathFloor(A/Tick)*Tick);
}  

//+-------------------------------------------------------------------------------------+
//|  ,                                                    |
//+-------------------------------------------------------------------------------------+
bool SetOrders()
{
// - 1 - ==        =====================================
 bool Buy = False, Sell = False;                                 //    
 double BuyStop = NP(Resis + Spread + Tick);//    BuyStop    Sell
 double SellStop = NP(Supp - Tick);         //    SellStop    Buy
 double BuyTake = NP(2*BuyStop - SellStop);                       //    Buy
 double SellTake = NP(2*SellStop - BuyStop);                     //    Sell
// - 1 -  ==   ============================================================

// - 2 - ==   / =================================================
 for (int i = OrdersTotal()-1; i >= 0; i--)
   if (OrderSelect(i, SELECT_BY_POS))
     if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
// - 2 -  ==   ============================================================

// - 3 - ==  BUY ==================================================================
       {
        RefreshRates();
        if (OrderType() == OP_BUY)
          {
           Buy = True;
           //   BUY   ,     
           if (ND(OrderStopLoss() - SellStop) != 0 && ND(Bid - StopLevel - SellStop) > 0
               && ND(BuyTake - Bid - StopLevel) > 0)
             if (WaitForTradeContext())
               if (!OrderModify(OrderTicket(), 0, SellStop, BuyTake, 0))
                 return(False);
          }       
// - 3 -  ==   ============================================================

// - 4 - ==  BUYSTOP ==============================================================
        if (OrderType() == OP_BUYSTOP)
          {
           Buy = True;
           //      ,   ,   
           if (ND(OrderOpenPrice() - BuyStop) != 0 && ND(BuyStop - Ask - StopLevel) > 0
               && ND(OrderOpenPrice() - Ask - FreezeLevel) > 0)
             if (WaitForTradeContext())
               if (!OrderModify(OrderTicket(), BuyStop, SellStop, BuyTake, 0))
                 return(False);
          }       
// - 4 -  ==   ============================================================

// - 5 - ==  SELL =================================================================
        if (OrderType() == OP_SELL)
          {
           Sell = True;
           if (ND(OrderStopLoss() - BuyStop) != 0 && ND(BuyStop - Ask - StopLevel) > 0 &&
               ND(Bid - StopLevel - SellTake) > 0)
             if (WaitForTradeContext())
               if (!OrderModify(OrderTicket(), 0, BuyStop, SellTake, 0))
                 return(False);
          }       
// - 5 -  ==   ============================================================

// - 6 - ==  SELLSTOP =============================================================
        if (OrderType() == OP_SELLSTOP)
          {
           Sell = True;
           if (ND(OrderOpenPrice() - SellStop) != 0 && ND(Bid - SellStop - StopLevel) > 0
               && ND(Bid - OrderOpenPrice() - FreezeLevel) > 0)
             if (WaitForTradeContext())
               if (!OrderModify(OrderTicket(), SellStop, BuyStop, SellTake, 0))
                 return(False);
          }       
       }
// - 6 - ===   ============================================================
       
// - 7 - == BuyStop  Buy   -  BuyStop ==============================
 if (!Buy && Ask < Resis)
   if (OpenOrderCorrect(OP_BUYSTOP, BuyStop, SellStop, BuyTake) > 0)
     return(False);
// - 7 - ===   ============================================================
 
// - 8 - == SellStop  Sell   -  SellStop ===========================
 if (!Sell && Bid > Supp)
   if (OpenOrderCorrect(OP_SELLSTOP, SellStop, BuyStop, SellTake) > 0)
     return(False);
// - 8 - ===   ============================================================
}
 
//+-------------------------------------------------------------------------------------+
//|  START                                                               |
//+-------------------------------------------------------------------------------------+
int start()
  {
// - 1 -  ==    ? ===========================================
   if (!Activate || FatalError)             //   ,  
    return(0);           //  init         
// - 1 -  ==   ============================================================
     
// - 2 - ==      ========================================
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                 //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //    
// - 2 -  ==   ============================================================
   
   if (LastBar == Time[0])
     return(0);
     
// - 3 - ==      ==================================
   GetLevels();
// - 3 -  ==   ============================================================

// - 4 - ==   ===========================================================
   if (Supp != 0 && Resis != 0)       //   ,   
     if (!SetOrders()) return(0);
// - 4 -  ==   ============================================================

   LastBar = Time[0];                                             //   ""

   return(0);
  }

